#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Representation of a recording of on-line handwritten data. On-line means
   that the pen trajectory is given (and not online as in 'Internet').
"""

import logging
import json
import matplotlib.pyplot as plt


class HandwrittenData(object):
    """Represents a handwritten symbol."""
    def __init__(self, raw_data_json, formula_id=None, raw_data_id=None,
                 formula_in_latex=None, wild_point_count=0,
                 missing_stroke=0, user_id=0):
        self.raw_data_json = raw_data_json
        self.formula_id = formula_id
        self.raw_data_id = raw_data_id
        self.formula_in_latex = formula_in_latex
        self.wild_point_count = wild_point_count
        self.missing_stroke = missing_stroke
        self.user_id = user_id
        assert type(json.loads(self.raw_data_json)) is list, \
            "raw_data_json is not JSON: %r" % self.raw_data_json
        assert len(self.get_pointlist()) >= 1, \
            "The pointlist of formula_id %i is %s" % (self.formula_id,
                                                      self.get_pointlist())
        assert wild_point_count >= 0
        assert missing_stroke >= 0

    def get_pointlist(self):
        """Get a list of lists of tuples from JSON raw data string.
           Those lists represent strokes with control points.

           Every point is a dictionary:
           {'x': 123, 'y': 42, 'time': 1337}
        """
        try:
            pointlist = json.loads(self.raw_data_json)
        except Exception as inst:  # pragma: no cover
            logging.debug("pointStrokeList: strokelistP")
            logging.debug(self.raw_data_json)
            logging.debug("didn't work")
            raise inst

        if len(pointlist) == 0:  # pragma: no cover
            logging.warning("Pointlist was empty. Search for '" +
                            self.raw_data_json + "' in `wm_raw_draw_data`.")
        return pointlist

    def get_sorted_pointlist(self):
        """Make sure that the points and strokes are in order."""
        pointlist = self.get_pointlist()
        for i in range(len(pointlist)):
            pointlist[i] = sorted(pointlist[i], key=lambda p: p['time'])
        pointlist = sorted(pointlist, key=lambda stroke: stroke[0]['time'])
        return pointlist

    def set_pointlist(self, pointlist):
        """Overwrite pointlist.
        :param pointlist: The inner lists represent strokes. Every stroke
            consists of points. Every point is a dictinary with 'x', 'y',
            'time'.
        :type pointlist: list of lists
        """
        assert type(pointlist) is list, \
            "pointlist is not of type list, but %r" % type(pointlist)
        assert len(pointlist) >= 1, \
            "The pointlist of formula_id %i is %s" % (self.formula_id,
                                                      self.get_pointlist())
        self.raw_data_json = json.dumps(pointlist)

    def get_bounding_box(self):
        """ Get the bounding box of a pointlist. """
        pointlist = self.get_pointlist()

        # Initialize bounding box parameters to save values
        minx, maxx = pointlist[0][0]["x"], pointlist[0][0]["x"]
        miny, maxy = pointlist[0][0]["y"], pointlist[0][0]["y"]
        mint, maxt = pointlist[0][0]["time"], pointlist[0][0]["time"]

        # Adjust parameters
        for stroke in pointlist:
            for p in stroke:
                minx, maxx = min(minx, p["x"]), max(maxx, p["x"])
                miny, maxy = min(miny, p["y"]), max(maxy, p["y"])
                mint, maxt = min(mint, p["time"]), max(maxt, p["time"])
        return {"minx": minx, "maxx": maxx, "miny": miny, "maxy": maxy,
                "mint": mint, "maxt": maxt}

    def get_width(self):
        """Get the width of the rectangular, axis-parallel bounding box."""
        box = self.get_bounding_box()
        return box['maxx'] - box['minx']

    def get_height(self):
        """Get the height of the rectangular, axis-parallel bounding box."""
        box = self.get_bounding_box()
        return box['maxy'] - box['miny']

    def get_area(self):
        """Get the area in square pixels of the recording."""
        return (self.get_height()+1) * (self.get_width()+1)

    def get_time(self):
        """Get the time in which the recording was created."""
        box = self.get_bounding_box()
        return box['maxt'] - box['mint']

    def preprocessing(self, algorithms):
        """Apply preprocessing algorithms.

        >>> a = HandwrittenData(...)
        >>> preprocessing_queue = [(preprocessing.scale_and_shift, []), \
                                   (preprocessing.connect_strokes, []), \
                                   (preprocessing.douglas_peucker, \
                                    {'EPSILON': 0.2}), \
                                   (preprocessing.space_evenly, \
                                    {'number': 100, \
                                     'KIND': 'cubic'})]
        >>> a.preprocessing(preprocessing_queue)
        """
        assert type(algorithms) is list
        for algorithm in algorithms:
            algorithm(self)

    def feature_extraction(self, algorithms):
        """Get a list of features.

        Every algorithm has to return the features as a list."""
        assert type(algorithms) is list
        features = []
        for algorithm in algorithms:
            new_features = algorithm(self)
            assert len(new_features) == algorithm.get_dimension(), \
                "Expected %i features from algorithm %s, got %i features" % \
                (algorithm.get_dimension(), str(algorithm), len(new_features))
            features += new_features
        return features

    def show(self):
        """Show the data graphically in a new pop-up window."""
        pointlist = self.get_pointlist()
        if 'pen_down' in pointlist[0][0]:
            assert len(pointlist) > 1, \
                "Lenght of pointlist was %i. Got: %s" % (len(pointlist),
                                                         pointlist)
            new_pointlist = []
            last = None
            stroke = []
            for point in pointlist[0]:
                if last is None:
                    last = point['pen_down']
                if point['pen_down'] != last:
                    new_pointlist.append(stroke)
                    last = point['pen_down']
                    stroke = []
                else:
                    stroke.append(point)
            new_pointlist.append(stroke)
            pointlist = new_pointlist

        _, ax = plt.subplots()
        ax.set_title("Raw data id: %s, "
                     "Formula_id: %s" % (str(self.raw_data_id),
                                         str(self.formula_id)))

        for stroke in pointlist:
            xs, ys = [], []
            for p in stroke:
                xs.append(p['x'])
                ys.append(p['y'])
            if "pen_down" in stroke[0] and stroke[0]["pen_down"] is False:
                plt.plot(xs, ys, '-x')
            else:
                plt.plot(xs, ys, '-o')
        plt.gca().invert_yaxis()
        ax.set_aspect('equal')
        plt.show()

    def count_single_dots(self):
        """Count all strokes of this recording that have only a single dot.
        """
        pointlist = self.get_pointlist()
        single_dots = 0
        for stroke in pointlist:
            if len(stroke) == 1:
                single_dots += 1
        return single_dots

    def get_center_of_mass(self):
        """Get a tuple (x,y) that is the center of mass. The center of mass
           is not necessarily the same as the center of the bounding box.
           Imagine a black square and a single dot wide outside of the square.
        """
        xsum, ysum, counter = 0., 0., 0
        for stroke in self.get_pointlist():
            for point in stroke:
                xsum += point['x']
                ysum += point['y']
                counter += 1
        return (xsum/counter, ysum/counter)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return "HandwrittenData(raw_data_id=%s)" % str(self.raw_data_id)

    def __str__(self):
        return self.__repr__()



#!/usr/bin/env python

import logging
import sys
logging.basicConfig(format="%(asctime)s %(levelname)s %(message)s",
                    level=logging.DEBUG,
                    stream=sys.stdout)
import time
import json
# GUI
from kivy.app import App
from kivy.uix.widget import Widget
from kivy.graphics import Color, Ellipse, Line
from kivy.logger import Logger
Logger.setLevel(logging.ERROR)
# mine
import hwrt.utils as utils


def unix_time():
    return int(round(time.time() * 1000))


recording = []
last_stroke = []


class MyPaintWidget(Widget):

    def on_touch_down(self, touch):
        global recording, last_stroke
        with self.canvas:
            Color(1, 1, 0)
            d = 5.
            Ellipse(pos=(touch.x - d / 2, touch.y - d / 2), size=(d, d))
            touch.ud['line'] = Line(points=(touch.x, touch.y))
            # Record keeping
            if len(last_stroke) > 0:
                recording.append(last_stroke)
                last_stroke = []
            point = {"x": touch.x, "y": -touch.y, "time": unix_time()}
            last_stroke.append(point)

    def on_touch_move(self, touch):
        global last_stroke
        touch.ud['line'].points += [touch.x, touch.y]
        # Record keeping
        point = {"x": touch.x, "y": -touch.y, "time": unix_time()}
        last_stroke.append(point)


class MyPaintApp(App):

    def build(self):
        return MyPaintWidget()


def get_parser():
    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-m", "--model",
                        dest="model",
                        help="where is the model folder (with a info.yml)?",
                        metavar="FOLDER",
                        type=lambda x: utils.is_valid_folder(parser, x),
                        default=utils.default_model())
    parser.add_argument("-v", "--verbose",
                        dest="verbose",
                        help="show recording before and after preprocessing",
                        action='store_true',
                        default=False)
    parser.add_argument("-n",
                        dest="n", default=10, type=int,
                        help="Show TOP-N results")
    return parser


if __name__ == '__main__':
    args = get_parser().parse_args()
    MyPaintApp().run()
    recording.append(last_stroke)
    last_stroke = []
    raw_data_json = json.dumps(recording)
    logging.info(recording)
    logging.info("Start evaluation...")
    # TODO: use args.verbose instead of True if possible
    results = utils.classify_single_recording(raw_data_json,
                                              args.model,
                                              True)
    n = args.n

    # Print headline
    print("{0:18s} {1:5s}".format("LaTeX Code", "Prob"))
    print("#"*50)
    for latex, probability in results:
        if n == 0:
            break
        else:
            n -= 1
        print("{0:18s} {1:5f}".format(latex, probability))
    print("#"*50)



#!/usr/bin/env python

"""Analyze data in a pickle file by maximum time / width / height and
   similar features.
"""

from __future__ import print_function
import os
import logging
import sys
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)
try:  # Python 2
    import cPickle as pickle
except ImportError:  # Python 3
    import pickle
import time
import numpy
from collections import defaultdict
import math
# My modules
# HandwrittenData is necessary because of pickle
from hwrt.HandwrittenData import HandwrittenData  # pylint: disable=unused-import
import hwrt.features as features
import hwrt.Geometry as Geometry
import hwrt.utils as utils


def sort_by_formula_id(raw_datasets):
    """"The parameter ``raw_datasets`` has to be of the format

    [{'is_in_testset': 0,
      'formula_id': 31L,
      'handwriting': HandwrittenData(raw_data_id=2953),
      'formula_in_latex': 'A',
      'id': 2953L},
     {'is_in_testset': 0,
      'formula_id': 31L,
      'handwriting': HandwrittenData(raw_data_id=4037),
      'formula_in_latex': 'A',
      'id': 4037L},
     {'is_in_testset': 0,
      'formula_id': 31L,
      'handwriting': HandwrittenData(raw_data_id=4056),
      'formula_in_latex': 'A',
      'id': 4056L}]
    """
    by_formula_id = defaultdict(list)
    for el in raw_datasets:
        by_formula_id[el['handwriting'].formula_id].append(el['handwriting'])
    return by_formula_id


def filter_label(label, replace_by_similar=True):
    """Some labels currently don't work together because of LaTeX naming
       clashes. Those will be replaced by simple strings. """
    bad_names = ['celsius', 'degree', 'ohm', 'venus', 'mars', 'astrosun',
                 'fullmoon', 'leftmoon', 'female', 'male', 'checked',
                 'diameter', 'sun', 'Bowtie', 'sqrt',
                 'cong', 'copyright', 'dag', 'parr', 'notin', 'dotsc',
                 'mathds', 'mathfrak']
    if any(label[1:].startswith(bad) for bad in bad_names):
        if label == '\\dag' and replace_by_similar:
            return '\\dagger'
        elif label == '\\diameter' and replace_by_similar:
            return '\\O'
        return label[1:]
    else:
        return label


def analyze_feature(raw_datasets, feature, basename="aspect_ratios"):
    """Apply 'feature' to all recordings in 'raw_datasets'. Store the
       results in two files. One file stores the raw result, the other one
       groups the results by symbols and stores the mean, standard deviation
       and the name of the symbol as a csv file.
    """
    # Get folder where results should get stored
    folder = os.path.join(utils.get_project_root(), "analyzation/")

    # Prepare files
    csv_filename = os.path.join(folder, basename+'.csv')
    raw_filename = os.path.join(folder, basename+'.raw')
    open(csv_filename, 'w').close()  # Truncate the file
    open(raw_filename, 'w').close()  # Truncate the file
    csv_file = open(csv_filename, 'a')
    raw_file = open(raw_filename, 'a')

    csv_file.write("label,mean,std\n")  # Write header
    raw_file.write("latex,raw_data_id,value\n")  # Write header

    by_formula_id = sort_by_formula_id(raw_datasets)
    print_data = []
    for _, datasets in by_formula_id.items():
        values = []
        for data in datasets:
            value = feature(data)[0]
            values.append(value)
            raw_file.write("%s,%i,%0.2f\n" % (datasets[0].formula_in_latex,
                                              data.raw_data_id,
                                              value))
        label = filter_label(datasets[0].formula_in_latex)
        print_data.append((label, numpy.mean(values), numpy.std(values)))

    # Sort the data by highest mean, descending
    print_data = sorted(print_data, key=lambda n: n[1], reverse=True)
    # Write data to file
    for label, mean, std in print_data:
        csv_file.write("%s,%0.2f,%0.2f\n" % (label, mean, std))
    csv_file.close()


def analyze_creator(raw_datasets, filename="creator.csv"):
    """Analyze who created most of the data."""

    # prepare file
    root = utils.get_project_root()
    folder = os.path.join(root, "analyzation/")
    workfilename = os.path.join(folder, filename)
    open(workfilename, 'w').close()  # Truncate the file
    write_file = open(workfilename, "a")
    write_file.write("creatorid,nr of symbols\n")  # heading

    print_data = defaultdict(int)
    start_time = time.time()
    for i, raw_dataset in enumerate(raw_datasets):
        if i % 100 == 0 and i > 0:
            utils.print_status(len(raw_datasets), i, start_time)
        print_data[raw_dataset['handwriting'].user_id] += 1
    print("\r100%"+"\033[K\n")
    # Sort the data by highest value, descending
    print_data = sorted(print_data.items(),
                        key=lambda n: n[1],
                        reverse=True)
    # Write data to file
    write_file.write("total,%i\n" % sum([value for _, value in print_data]))
    for userid, value in print_data:
        write_file.write("%i,%i\n" % (userid, value))
    write_file.close()


def analyze_instroke_speed(raw_datasets, filename="instroke_speed.csv"):
    """Analyze how fast the points were in pixel/ms."""
    # prepare file
    folder = os.path.join(utils.get_project_root(), "analyzation/")
    workfilename = os.path.join(folder, filename)
    open(workfilename, 'w').close()  # Truncate the file
    write_file = open(workfilename, "a")
    write_file.write("speed\n")  # heading

    print_data = []
    start_time = time.time()
    for i, raw_dataset in enumerate(raw_datasets):
        if i % 100 == 0 and i > 0:
            utils.print_status(len(raw_datasets), i, start_time)
        pointlist = raw_dataset['handwriting'].get_sorted_pointlist()

        for stroke in pointlist:
            for last_point, point in zip(stroke, stroke[1:]):
                space_dist = math.hypot(last_point['x'] - point['x'],
                                        last_point['y'] - point['y'])
                time_delta = point['time'] - last_point['time']
                if time_delta == 0:
                    continue
                print_data.append(space_dist/time_delta)
    print("\r100%"+"\033[K\n")
    # Sort the data by highest value, descending
    print_data = sorted(print_data, reverse=True)
    # Write data to file
    for value in print_data:
        write_file.write("%0.8f\n" % (value))

    logging.info("instroke speed mean: %0.8f", numpy.mean(print_data))
    logging.info("instroke speed std: %0.8f", numpy.std(print_data))
    write_file.close()


def analyze_distance_betwee_strokes(raw_datasets,
                                    filename="dist_between_strokes.csv"):
    """Analyze how much distance in px is between strokes."""
    # prepare file
    root = utils.get_project_root()
    folder = os.path.join(root, "analyzation/")
    workfilename = os.path.join(folder, filename)
    open(workfilename, 'w').close()  # Truncate the file
    write_file = open(workfilename, "a")
    write_file.write("speed\n")  # heading

    print_data = []
    start_time = time.time()
    for i, raw_dataset in enumerate(raw_datasets):
        if i % 100 == 0 and i > 0:
            utils.print_status(len(raw_datasets), i, start_time)
        pointlist = raw_dataset['handwriting'].get_sorted_pointlist()

        for last_stroke, stroke in zip(pointlist, pointlist[1:]):
            p1 = last_stroke[-1]
            p2 = stroke[0]
            space_dist = math.hypot(p1['x'] - p2['x'],
                                    p1['y'] - p2['y'])
            print_data.append(space_dist)
    print("\r100%"+"\033[K\n")
    # Sort the data by highest value, descending
    print_data = sorted(print_data, reverse=True)
    # Write data to file
    for value in print_data:
        write_file.write("%0.8f\n" % (value))

    logging.info("dist_between_strokes mean:\t%0.8fpx", numpy.mean(print_data))
    logging.info("dist_between_strokes std: \t%0.8fpx", numpy.std(print_data))
    write_file.close()


def get_bounding_box_distance(raw_datasets):
    """Get the distances between bounding boxes of strokes of a single symbol.
       Can only be applied to recordings with at least two strokes.
    """

    # TODO: Deal with http://www.martin-thoma.de/write-math/symbol/?id=167
    # 193
    # 524

    def _get_stroke_bounding_box(stroke):
        """Get the bounding box of a stroke. A stroke is a list of points
           {'x': 123, 'y': 456, 'time': 42} and a bounding box is the smallest
           rectangle that contains all points.
        """
        min_x, max_x = stroke[0]['x'], stroke[0]['x']
        min_y, max_y = stroke[0]['y'], stroke[0]['y']
        #  if len(stroke) == 1: ?
        for point in stroke:
            min_x = min(point['x'], min_x)
            max_x = max(point['x'], max_x)
            min_y = min(point['y'], min_y)
            max_y = max(point['y'], max_y)
        minp = Geometry.Point(min_x, min_y)
        maxp = Geometry.Point(max_x, max_y)
        return Geometry.BoundingBox(minp, maxp)

    def _get_bb_distance(a, b):
        """"Take two bounding boxes a and b and get the smallest distance
            between them.
        """
        points_a = [Geometry.Point(a.p1.x, a.p1.y),
                    Geometry.Point(a.p1.x, a.p2.y),
                    Geometry.Point(a.p2.x, a.p1.y),
                    Geometry.Point(a.p2.x, a.p2.y)]
        points_b = [Geometry.Point(b.p1.x, b.p1.y),
                    Geometry.Point(b.p1.x, b.p2.y),
                    Geometry.Point(b.p2.x, b.p1.y),
                    Geometry.Point(b.p2.x, b.p2.y)]
        min_distance = points_a[0].dist_to(points_b[0])
        for pa in points_a:
            for pb in points_b:
                min_distance = min(min_distance, pa.dist_to(pb))
        lines_a = [Geometry.LineSegment(points_a[0], points_a[1]),
                   Geometry.LineSegment(points_a[1], points_a[2]),
                   Geometry.LineSegment(points_a[2], points_a[3]),
                   Geometry.LineSegment(points_a[3], points_a[0])]
        lines_b = [Geometry.LineSegment(points_b[0], points_b[1]),
                   Geometry.LineSegment(points_b[1], points_b[2]),
                   Geometry.LineSegment(points_b[2], points_b[3]),
                   Geometry.LineSegment(points_b[3], points_b[0])]
        for line_in_a in lines_a:
            for line_in_b in lines_b:
                min_distance = min(min_distance, line_in_a.dist_to(line_in_b))
        return min_distance

    bbfile = open("bounding_boxdist.html", "a")
    start_time = time.time()
    for i, raw_dataset in enumerate(raw_datasets):
        if i % 100 == 0 and i > 0:
            utils.print_status(len(raw_datasets), i, start_time)
        pointlist = raw_dataset['handwriting'].get_pointlist()
        if len(pointlist) < 2:
            continue
        bounding_boxes = []
        for stroke in pointlist:
            # TODO: Get bounding boxes of strokes
            bounding_boxes.append(_get_stroke_bounding_box(stroke))

        got_change = True
        while got_change:
            got_change = False
            i = 0
            while i < len(bounding_boxes):
                j = i + 1
                while j < len(bounding_boxes):
                    if Geometry.do_bb_intersect(bounding_boxes[i],
                                                bounding_boxes[j]):
                        got_change = True
                        new_bounding_boxes = []
                        p1x = min(bounding_boxes[i].p1.x,
                                  bounding_boxes[j].p1.x)
                        p1y = min(bounding_boxes[i].p1.y,
                                  bounding_boxes[j].p1.y)
                        p2x = max(bounding_boxes[i].p2.x,
                                  bounding_boxes[j].p2.x)
                        p2y = max(bounding_boxes[i].p2.y,
                                  bounding_boxes[j].p2.y)
                        p1 = Geometry.Point(p1x, p1y)
                        p2 = Geometry.Point(p2x, p2y)
                        new_bounding_boxes.append(Geometry.BoundingBox(p1, p2))
                        for k in range(len(bounding_boxes)):
                            if k != i and k != j:
                                new_bounding_boxes.append(bounding_boxes[k])
                        bounding_boxes = new_bounding_boxes
                    j += 1
                i += 1

        # sort bounding boxes (decreasing) by size
        bounding_boxes = sorted(bounding_boxes,
                                key=lambda bbox: bbox.get_area(),
                                reverse=True)

        # Bounding boxes have been merged as far as possible
        # check their distance and compare it with the highest dimension
        # (length/height) of the biggest bounding box
        if len(bounding_boxes) != 1:
            bb_dist = []
            for k, bb in enumerate(bounding_boxes):
                dist_tmp = []
                for j, bb2 in enumerate(bounding_boxes):
                    if k == j:
                        continue
                    dist_tmp.append(_get_bb_distance(bb, bb2))
                bb_dist.append(min(dist_tmp))
            bb_dist = max(bb_dist)
            dim = max([bb.get_largest_dimension() for bb in bounding_boxes])
            if bb_dist > 1.5*dim:
                # bounding_box_h = raw_dataset['handwriting'].get_bounding_box()
                # bbsize = (bounding_box_h['maxx'] - bounding_box_h['minx']) * \
                #          (bounding_box_h['maxy'] - bounding_box_h['miny'])
                if raw_dataset['handwriting'].formula_id not in \
                   [635, 636, 936, 992, 260, 941, 934, 184] and \
                   raw_dataset['handwriting'].wild_point_count == 0 and \
                   raw_dataset['handwriting'].missing_stroke == 0:
                    # logging.debug("bb_dist: %0.2f" % bb_dist)
                    # logging.debug("dim: %0.2f" % dim)
                    # for bb in bounding_boxes:
                    #     print(bb)
                    #     print("width: %0.2f" % bb.get_width())
                    #     print("height: %0.2f" % bb.get_height())
                    #     print("maxdim: %0.2f" % bb.get_largest_dimension())
                    # bb_dist = []
                    # for k, bb in enumerate(bounding_boxes):
                    #     dist_tmp = []
                    #     for j, bb2 in enumerate(bounding_boxes):
                    #         if k == j:
                    #             continue
                    #         dist_tmp.append(_get_bb_distance(bb, bb2))
                    #     print(dist_tmp)
                    #     bb_dist.append(min(dist_tmp))
                    # raw_dataset['handwriting'].show()
                    # exit()
                    url_base = "http://www.martin-thoma.de/write-math/view"
                    bbfile.write("<a href='%s/?raw_data_id=%i'>a</a>\n" %
                                 (url_base,
                                  raw_dataset['handwriting'].raw_data_id))
    print("\r100%"+"\033[K\n")


def get_time_between_controll_points(raw_datasets):
    """For each recording: Store the average time between controll points of
       one stroke / controll points of two different strokes.
    """
    average_between_points = open("average_time_between_points.txt", "a")
    average_between_strokes = open("average_time_between_strokes.txt", "a")
    start_time = time.time()
    for i, raw_dataset in enumerate(raw_datasets):
        if i % 100 == 0 and i > 0:
            utils.print_status(len(raw_datasets), i, start_time)

        # Do the work
        times_between_points, times_between_strokes = [], []
        last_stroke_end = None
        if len(raw_dataset['handwriting'].get_pointlist()) == 0:
            logging.warning("%i has no content.",
                            raw_dataset['handwriting'].raw_data_id)
            continue
        for stroke in raw_dataset['handwriting'].get_sorted_pointlist():
            if last_stroke_end is not None:
                times_between_strokes.append(stroke[-1]['time'] -
                                             last_stroke_end)
            last_stroke_end = stroke[-1]['time']
            last_point_end = None
            for point in stroke:
                if last_point_end is not None:
                    times_between_points.append(point['time'] - last_point_end)
                last_point_end = point['time']
        # The recording might only have one point
        if len(times_between_points) > 0:
            average_between_points.write("%0.2f\n" %
                                         numpy.average(times_between_points))
        # The recording might only have one stroke
        if len(times_between_strokes) > 0:
            average_between_strokes.write("%0.2f\n" %
                                          numpy.average(times_between_strokes))
    print("\r100%"+"\033[K\n")
    average_between_points.close()
    average_between_strokes.close()


def main(handwriting_datasets_file):
    """Start the creation of the wanted metric."""
    # Load from pickled file
    logging.info("Start loading data '%s' ...", handwriting_datasets_file)
    loaded = pickle.load(open(handwriting_datasets_file))
    raw_datasets = loaded['handwriting_datasets']
    logging.info("%i datasets loaded.", len(raw_datasets))
    logging.info("Start analyzing...")
    # logging.info("get_time_between_controll_points...")
    # get_time_between_controll_points(raw_datasets)
    # logging.info("get_bounding_box_distance...")
    # get_bounding_box_distance(raw_datasets)

    featurelist = [(features.AspectRatio(), "aspect_ratio.csv"),
                   (features.ReCurvature(1), "re_curvature.csv"),
                   (features.Height(), "height.csv"),
                   (features.Width(), "width.csv"),
                   (features.Time(), "time.csv"),
                   (features.Ink(), "ink.csv"),
                   (features.StrokeCount(), "stroke-count.csv")]
    for feat, filename in featurelist:
        logging.info("create %s...", filename)
        analyze_feature(raw_datasets, feat, filename)

    logging.info("creator...")
    analyze_creator(raw_datasets)
    analyze_instroke_speed(raw_datasets)
    analyze_distance_betwee_strokes(raw_datasets)


def get_parser():
    """Return the parser object for this script."""
    project_root = utils.get_project_root()

    # Get latest (raw) dataset
    dataset_folder = os.path.join(project_root, "raw-datasets")
    latest_dataset = utils.get_latest_in_folder(dataset_folder, "raw.pickle")

    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-d", "--handwriting_datasets",
                        dest="handwriting_datasets",
                        help="where are the pickled handwriting_datasets?",
                        metavar="FILE",
                        type=lambda x: utils.is_valid_file(parser, x),
                        default=latest_dataset)
    return parser


if __name__ == '__main__':
    args = get_parser().parse_args()
    main(args.handwriting_datasets)
